/*****************************  Globals   *************************************/
const nsCoxMailPOPClassID = Components.ID("{4be51600-cd4e-11dc-95ff-0800200c9a66}");
const nsCoxMailPOPContactID = "@mozilla.org/CoxMailPOP;1";
const ExtCoxMailGuid = "{891cc720-cd4e-11dc-95ff-0800200c9a66}";

const szVersionNumber = "V20080128100000";

/******************************  CoxMail ***************************************/


function nsCoxMail()
{
    try
    {
        var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                                      .getService(Components.interfaces.mozIJSSubScriptLoader);
        scriptLoader.loadSubScript("chrome://web-mail/content/common/DebugLog.js");
        scriptLoader.loadSubScript("chrome://web-mail/content/common/CommonPrefs.js");
        scriptLoader.loadSubScript("chrome://web-mail/content/common/HttpComms3.js");
        scriptLoader.loadSubScript("chrome://coxmail/content/CoxMail-POPMSG.js");

        var date = new Date();
        var szLogFileName = "CoxMailLog_POP_" + date.getHours()+ "_" + date.getMinutes() + "_"+ date.getUTCMilliseconds() +"_";
        this.m_Log = new DebugLog("webmail.logging.comms", ExtCoxMailGuid, szLogFileName);

        this.m_Log.Write("nsCoxMailPOP.js " + szVersionNumber + " - Constructor - START");

        if (typeof PATTERN_COX_MAIL_CONSTANTS == "undefined")
        {
            this.m_Log.Write("nsCoxMailPOP.js - Constructor - loading constants");
            scriptLoader.loadSubScript("chrome://coxmail/content/CoxMail-Constants.js");
        }

        this.m_bAuthorised = false;
        this.m_bPopAccessible = true;
        this.m_szUserName = null;
        this.m_szPassWord = null;
        this.m_oResponseStream = null;
        this.m_HttpComms = new HttpComms(this.m_Log);
        this.m_HttpComms.setUserAgentOverride(true);
        
        this.m_iStage = 0;
        this.m_iTotalSize = 0;
        this.m_aCoxMailPopMessages = null;
        this.m_szMsgID = 0;

        this.m_ComponentManager = Components.classes["@mozilla.org/ComponentData2;1"]
                                            .getService(Components.interfaces.nsIComponentData2);

        this.m_IOService = Components.classes["@mozilla.org/network/io-service;1"];
        this.m_IOService = this.m_IOService.getService(Components.interfaces.nsIIOService);

        //do i reuse the session
        this.m_bReEntry = false;
        var oPref = {Value:null};
        var  webMailPrefAccess = new WebMailCommonPrefAccess();
        if (webMailPrefAccess.Get("bool","coxmail.bReUseSession",oPref))
            this.m_bReUseSession = oPref.Value;
        else
            this.m_bReUseSession = false;
        this.m_Log.Write("nsCoxMailPOP.js - Constructor - bReUseSession : " + this.m_bReUseSession);

        //do i download unread msg only
        if (webMailPrefAccess.Get("bool","coxmail.bDownloadUnread",oPref))
            this.m_bDownloadUnread = oPref.Value;
        else
            this.m_bDownloadUnread = true;
        this.m_Log.Write("nsCoxMailPOP.js - Constructor - bDownloadUnread : " + this.m_bDownloadUnread);

        //retrieve name of pop server
        if (webMailPrefAccess.Get("char","coxmail.szPopServer",oPref))
            this.m_szPopServer = oPref.Value;
        else
            this.m_szPopServer = "pop.central.cox.net";
        this.m_Log.Write("nsCoxMailPOP.js - Constructor - szPopServer : " + this.m_szPopServer);

        this.m_Log.Write("nsCoxMailPOP.js - Constructor - END");
    }
    catch(e)
    {
        this.m_Log.DebugDump("nsCoxMailPOP.js: Constructor : Exception : "
                                      + e.name
                                      + ".\nError message: "
                                      + e.message
                                      + ".\nException: "
                                      + e);
    }
}



nsCoxMail.prototype =
{
    get userName() {return this.m_szUserName;},
    set userName(userName) {return this.m_szUserName = userName;},

    get passWord() {return this.m_szPassWord;},
    set passWord(passWord) {return this.m_szPassWord = passWord;},

    get bAuthorised() {return this.m_bAuthorised;},

    get ResponseStream() {return this.m_oResponseStream;},
    set ResponseStream(responseStream) {return this.m_oResponseStream = responseStream;},


    logIn : function()
    {
        try
        {
            this.m_Log.Write("nsCoxMailPOP.js - logIN - START - Username: " + this.m_szUserName
                                                   + " Password: " + this.m_szPassWord
                                                   + " stream: " + this.m_oResponseStream);

            if (!this.m_szUserName || !this.m_oResponseStream  || !this.m_szPassWord) return false;

            this.m_HttpComms.setUserName(this.m_szUserName);

            var bSessionStored = this.m_ComponentManager.findElement(this.m_szUserName, "bSessionStored");
            if ( bSessionStored && this.m_bReUseSession ) 
            {
                    this.m_Log.Write("nsCoxMailPOP.js - logIN - Session Data found");
                    this.m_bReEntry = true;
    
                    this.serverComms("+OK Session Found\r\n");
                    this.m_bAuthorised = true;
            } 
            else 
            {
                this.m_Log.Write("nsCoxMailPOP.js - logIn - No Session Data found");
                var oCookies = Components.classes["@mozilla.org/nsWebMailCookieManager2;1"]
                                         .getService(Components.interfaces.nsIWebMailCookieManager2);
                oCookies.removeCookie(this.m_szUserName);

                this.m_ComponentManager.deleteAllElements(this.m_szUserName);
                
                // try connecting to pop server
                this.m_Log.Write("nsCoxMailPOP.js - logIn - attempting to connect to pop server: " + this.m_szPopServer);
			    var transportService =
					Components.classes["@mozilla.org/network/socket-transport-service;1"]
				        .getService(Components.interfaces.nsISocketTransportService);
			    var transport = transportService.createTransport(null, 0, this.m_szPopServer, 110, null);
			    transport.setTimeout(0,5); // connect timeout of 5 seconds
			    transport.setTimeout(1,5); // read-write timeout of 5 seconds

			    var outstream = transport.openOutputStream(0,0,0);

			    var stream = transport.openInputStream(0,0,0);
    			var instream = Components.classes["@mozilla.org/scriptableinputstream;1"]
      				.createInstance(Components.interfaces.nsIScriptableInputStream);
    			instream.init(stream);
    			
    			var mainObject = this;
    
			    var popListener = {
			    	
			    	onStartRequest: function(request, context){
						mainObject.m_Log.Write("nsCoxMailPOP.js - popListener - onStartRequest");
      				},
      				
      				onStopRequest: function(request, context, status){
						mainObject.m_Log.Write("nsCoxMailPOP.js - popListener - onStopRequest - status: " + status);
        				instream.close();
//        				outstream.close();
        				// check for status of 0 (success) or 8000FFFF (connection reset by peer)
        				if ( (status == 0) || (status == 2147549183) ) {
							mainObject.m_Log.Write("nsCoxMailPOP.js - popListener - onStopRequest - pop is accessible");
        					mainObject.m_bPopAccessible = true;
		                    mainObject.serverComms("+OK POP accessible\r\n");
                    		mainObject.m_bAuthorised = true;
        				} else {
        					// if pop is not accessible, proceed with login
        					mainObject.m_bPopAccessible = false;
			                mainObject.m_HttpComms.addValuePair('client', encodeURIComponent("html"));
            			    mainObject.m_HttpComms.addValuePair('login', encodeURIComponent("1"));
                			mainObject.m_HttpComms.addValuePair('targetFN', encodeURIComponent("WebMail"));
                			mainObject.m_HttpComms.addValuePair('rememberme', encodeURIComponent("false"));
                			mainObject.m_HttpComms.addValuePair('username', encodeURIComponent(mainObject.m_szUserName));
                			mainObject.m_HttpComms.addValuePair('emaildomain', encodeURIComponent("@cox.net"));
                			mainObject.m_HttpComms.addValuePair('password', encodeURIComponent(mainObject.m_szPassWord));
                			mainObject.m_HttpComms.addValuePair('onsuccess', encodeURIComponent(COX_MAIL_URL));
                			mainObject.m_HttpComms.addValuePair('onfailure', encodeURIComponent(COX_LOGIN_FAILURE_URL));

                			// mainObject.m_HttpComms.m_bHandleBounce  = false

                			mainObject.m_HttpComms.setURI(COX_LOGIN_URL);
                			mainObject.m_HttpComms.setRequestMethod("POST");

                			mainObject.m_iStage = 0;
                			var bResult = mainObject.m_HttpComms.send(mainObject.loginOnloadHandler, mainObject);
                			if (!bResult) { //throw new Error('httpConnection returned false');
								mainObject.m_Log.Write("nsCoxMailPOP.js - popListener - onStopRequest - bResult: " + bResult);
		                    	mainObject.serverComms("-ERR httpConnection returned false\r\n");
                    			mainObject.m_bAuthorised = false;
                    		}
        				}
        					
				    },

				    onDataAvailable: function(request, context, inputStream, offset, count){
				    	var inputData = instream.read(count);
						mainObject.m_Log.Write("nsCoxMailPOP.js - popListener - onDataAvailable data: " + inputData);
						if (inputData == "+OK InterMail POP3 server ready.\r\n") {
						    var outputData = "QUIT\r\n";
							mainObject.m_Log.Write("nsCoxMailPOP.js - popListener - onDataAvailable sending: " + outputData);
    						outstream.write(outputData,outputData.length);
    					}
        				
      				},
    			};

    			var pump = Components.
      							classes["@mozilla.org/network/input-stream-pump;1"].
        						createInstance(Components.interfaces.nsIInputStreamPump);
    			pump.init(stream, -1, -1, 0, 0, false);
    			pump.asyncRead(popListener,null);
                
            }
            this.m_Log.Write("nsCoxMailPOP.js - logIN - END");
            return true;
        }
        catch(e)
        {
            this.m_Log.DebugDump("nsCoxMailPOP.js: logIN : Exception : "
                                              + e.name +
                                              ".\nError message: "
                                              + e.message);
            return false;
        }
    },


    loginOnloadHandler : function(szResponse ,event , mainObject)
    {
        try 
        {
            mainObject.m_Log.Write("nsCoxMailPOP.js - loginOnloadHandler - START");
            mainObject.m_Log.Write("nsCoxMailPOP.js - loginOnloadHandler : stage " + mainObject.m_iStage);
    
            var httpChannel = event.QueryInterface(Components.interfaces.nsIHttpChannel);
    
            //if this fails we've gone somewhere new
            var responseStatus = httpChannel.responseStatus;
            if (!responseStatus) throw new Error("missing response status");
            mainObject.m_Log.Write("nsCoxMailPOP.js - loginOnloadHandler - status :" + responseStatus );
            if (responseStatus != 200)
                throw new Error("return status " + responseStatus);

            var aRedirect = szResponse.match(PATTERN_COX_MAIL_LOGIN_REDIRECT);
            if ( !aRedirect ) {
                this.m_bReEntry = false;
                throw new Error("Login failed. Redirect not found");
            }
        
            var szURI = decodeURIComponent(aRedirect[1]);
            if ( szURI != COX_MAIL_URL ) {
                this.m_bReEntry = false;
                throw new Error("Login failed. Wrong Userid/Password??");
            }
        
            mainObject.m_Log.Write("nsCoxMailPOP.js - loginOnloadHandler - redirectURL " + szURI);
     
            mainObject.serverComms("+OK login successful\r\n");
            mainObject.m_bAuthorised = true;

            mainObject.m_Log.Write("nsCoxMailPOP.js - loginOnloadHandler - END");
        }
        catch(err)
        {
            var oCookies = Components.classes["@mozilla.org/nsWebMailCookieManager2;1"]
                                     .getService(Components.interfaces.nsIWebMailCookieManager2);
            oCookies.removeCookie(mainObject.m_szUserName);

            mainObject.m_ComponentManager.deleteAllElements(mainObject.m_szUserName);

            mainObject.m_Log.DebugDump("nsCoxMailPOP.js: loginHandler : Exception : "
                                          + err.name
                                          + ".\nError message: "
                                          + err.message);

            mainObject.serverComms("-ERR negative vibes (CoxMail)\r\n");
        }
    },



    //stat
    //total size is in octets
    getNumMessages : function()
    {
        try
        {
        	var bResult = false;
            this.m_Log.Write("nsCoxMailPOP.js - getNumMessages - START");

			if (this.m_bPopAccessible) {
        	    this.serverComms("+OK 0 0\r\n");
				return true;
			}

            if ( this.m_bAuthorised == false ) return false;
          
            this.m_HttpComms.setURI(COX_MAIL_URL);
            this.m_HttpComms.setRequestMethod("GET");
            var bResult = this.m_HttpComms.send(this.mailBoxOnloadHandler, this);
            if (!bResult) throw new Error("httpConnection returned false"); 
	        this.m_aCoxMailPopMessages = new Array();
            this.m_iStage = 0;

            this.m_Log.Write("nsCoxMailPOP.js - getNumMessages - END - returning " + bResult);
            return bResult;
        }
        catch(e)
        {
            this.m_Log.DebugDump("nsCoxMailPOP.js: getNumMessages : Exception : "
                                          + e.name
                                          + ".\nError message: "
                                          + e.message);
            return false;
        }
    },




    mailBoxOnloadHandler : function (szResponse ,event , mainObject)
    {
        try
        {
            mainObject.m_Log.Write("nsCoxMailPOP.js - mailBoxOnloadHandler - START");
            mainObject.m_Log.Write("nsCoxMailPOP.js - mailBoxOnloadHandler : stage " + mainObject.m_iStage);

            var httpChannel = event.QueryInterface(Components.interfaces.nsIHttpChannel);

            //check status should be 200.
            var responseStatus = httpChannel.responseStatus;
            if (!responseStatus) throw new Error("missing response status");
            mainObject.m_Log.Write("nsCoxMailPOP.js - mailBoxOnloadHandler - Mailbox : status " + responseStatus);
            if (responseStatus != 200 )
                throw new Error("error status " + responseStatus);

            if ( szResponse.indexOf(COX_LOGIN_URL) > -1) {
            	throw new Error("LoginURL found on Mail page");
            }

            // now find the message IDs
            var bMoreMessagesExist = (szResponse.indexOf('title="Already on Last Page"') < 0);
            var szMailTable = szResponse.match(PATTERN_COX_MAIL_MESSAGE_TABLE);
            //mainObject.m_Log.Write("nsCoxMailPOP.js - mailBoxOnloadHandler - mail table : " + szMailTable);
            var aszMessageRows = szMailTable[0].match(PATTERN_COX_MAIL_MESSAGE_ROW);
		    //mainObject.m_Log.Write("nsCoxMailPOP.js - mailBoxOnloadHandler - message rows : " + aszMessageRows);
        	if (aszMessageRows) {
        		for (var i=1; i<aszMessageRows.length; i++) {
        			szMessageRow = aszMessageRows[i];
				    mainObject.m_Log.Write("nsCoxMailPOP.js - mailBoxOnloadHandler - message row : " + szMessageRow);
					//mainObject.m_Log.Write("nsCoxMailPOP.js - " + szMessageRow.match(/<tr>[\s]*<td.*?>[\s]*No Messages[\s]*<\/td>[\s]*<\/tr>/gm));
				    if (szMessageRow.match(PATTERN_NO_MESSAGES_ROW)) {
					    mainObject.m_Log.Write("nsCoxMailPOP.js - mailBoxOnloadHandler - No Messages");
					    bMoreMessagesExist = false;
				    } else {
					    var aszMessageId = szMessageRow.match(PATTERN_MESSAGE_ID)[1];
					    mainObject.m_Log.Write("nsCoxMailPOP.js - mailBoxOnloadHandler - message id : " + aszMessageId[1]);
					    var aszFmtSpace = szMessageRow.match(PATTERN_MESSAGE_SIZE);
					    //mainObject.m_Log.Write("nsCoxMailPOP.js - mailBoxOnloadHandler - aszFmtSpace : " + aszFmtSpace);
					    var iMessageSize = aszFmtSpace[1];
				    	var szUnit = aszFmtSpace[2];
					    if (szUnit == "KB") {
					    	iMessageSize = iMessageSize * 1024;
					    } else if (szUnit == "MB") {
					    	iMessageSize = iMessageSize * 1048576;
						}				    	
				    	mainObject.m_Log.Write("nsCoxMailPOP.js - mailBoxOnloadHandler - message size : " + iMessageSize);
				    
					    var coxMailPopMessage = new CoxMailPopMessage();
					    coxMailPopMessage.szMessageID = aszMessageId;
					    coxMailPopMessage.iSize = iMessageSize;
					    mainObject.m_aCoxMailPopMessages[mainObject.m_aCoxMailPopMessages.length] = coxMailPopMessage;
					}
				}
			} else {
				throw new Error("Unable to find message row");
			}
        			
        	if (bMoreMessagesExist) {
            	// get the next page
            	mainObject.m_iStage++;
	            mainObject.m_HttpComms.setURI(COX_MAIL_URL + "?page=" + mainObject.m_iStage);
    	        mainObject.m_HttpComms.setRequestMethod("GET");
        	    var bResult = mainObject.m_HttpComms.send(mainObject.mailBoxOnloadHandler, mainObject);
            	if (!bResult) throw new Error("httpConnection returned false"); 
            } else {
        		// last page
        		var iTotalSize = 0;
        		for (var i=0; i<mainObject.m_aCoxMailPopMessages.length; i++) {
        			iTotalSize = iTotalSize + mainObject.m_aCoxMailPopMessages[i].iSize;
        		}
        		mainObject.m_iTotalSize = iTotalSize;
	            var szAnswer = "+OK " + mainObject.m_aCoxMailPopMessages.length + " " + iTotalSize;
    	        mainObject.m_Log.Write("nsCoxMailPOP.js - responding " + szAnswer);
        	    mainObject.serverComms(szAnswer + "\r\n");
                mainObject.m_bAuthorised = true;
            }

            mainObject.m_Log.Write("nsCoxMailPOP.js - MailBoxOnload - END");
        }
        catch(err)
        {
             mainObject.m_Log.DebugDump("nsCoxMailPOP.js: MailboxOnload : Exception : "
                                              + err.name
                                              + ".\nError message: "
                                              + err.message
                                              + "\n" + err.lineNumber);

             mainObject.serverComms("-ERR negative vibes (CoxMail)\r\n");
        }
    },





    //list
    //I'm not downloading the mailbox again.
    //I hope stat been called first or there's going to be trouble
    getMessageSizes : function()
    {
        try
        {
            this.m_Log.Write("nsCoxMailPOP.js - getMessageSizes - START");

			if (this.m_bPopAccessible) {
        	    this.serverComms("+OK 0 0\r\n");
				return true;
			}

            if ( this.m_bAuthorised == false ) return false;
            
            var aCoxMailPopMessages = this.m_aCoxMailPopMessages;
            
            var szAnswer = "+OK " + aCoxMailPopMessages.length + " " + this.m_iTotalSize + "\r\n";
            
            for (var i=0; i<aCoxMailPopMessages.length; i++) {
            	szAnswer = szAnswer + (i+1) + " " + aCoxMailPopMessages[i].iSize + "\r\n";
            }
            
   	        this.m_Log.Write("nsCoxMailPOP.js - getMessageSizes responding " + szAnswer);
        	this.serverComms(szAnswer + ".\r\n");
            this.m_bAuthorised = true;

            this.m_Log.Write("nsCoxMailPOP.js - getMessageSizes - END");
            return true;
        }
        catch(e)
        {
            this.m_Log.DebugDump("nsCoxMailPOP.js: getMessageSizes : Exception : "
                        + e.name
                        + ".\nError message: "
                        + e.message + "\n"
                        + e.lineNumber);
            return false;
        }
    },




    //IUDL
    getMessageIDs : function()
    {
        try
        {
            this.m_Log.Write("nsCoxMailPOP.js - getMessageIDs - START");

			if (this.m_bPopAccessible) {
        	    this.serverComms("+OK 0 0\r\n");
				return true;
			}

            if ( this.m_bAuthorised == false ) return false;

            var aCoxMailPopMessages = this.m_aCoxMailPopMessages;
            
            var szAnswer = "+OK " + aCoxMailPopMessages.length + " " + this.m_iTotalSize + "\r\n";
            
            for (var i=0; i<aCoxMailPopMessages.length; i++) {
            	szAnswer = szAnswer + (i+1) + " " + aCoxMailPopMessages[i].szMessageID + "\r\n";
            }
            
   	        this.m_Log.Write("nsCoxMailPOP.js - getMessageSizes responding " + szAnswer);
        	this.serverComms(szAnswer + ".\r\n");
            this.m_bAuthorised = true;

            this.m_Log.Write("nsCoxMailPOP.js - getMessageIDs - END");
            return true;
        }
        catch(e)
        {
            this.m_Log.DebugDump("nsCoxMailPOP.js: getMessageIDs : Exception : "
                                              + e.name
                                              + ".\nError message: "
                                              + e.message);
            return false;
        }
    },




    //retr
    getMessage : function(msgIndex)
    {
        try
        {
            this.m_Log.Write("nsCoxMailPOP.js - getMessage - START");
    
            if ( this.m_bAuthorised == false ) return false;
    
            //get msg id
            this.m_szMsgID = this.m_aCoxMailPopMessages[msgIndex-1].szMessageID;
            this.m_Log.Write("nsCoxMailPOP.js - getMessage - msg id: " + this.m_szMsgID);
    
            this.m_iStage = 0;
            this.m_HttpComms.setURI(COX_MAIL_VIEW_URL + '?msgId=' + this.m_szMsgID);
            this.m_HttpComms.setRequestMethod("GET");
            var bResult = this.m_HttpComms.send(this.emailOnloadHandler, this);
            if (!bResult) throw new Error("httpConnection returned false");
    
            this.m_Log.Write("nsCoxMailPOP.js - getMessage - END");
            return true;
        }
        catch(e)
        {
             this.m_Log.DebugDump("nsCoxMailPOP.js: getMessage : Exception : "
                                          + e.name +
                                          ".\nError message: "
                                          + e.message);
            return false;
        }
    },


    emailOnloadHandler : function (szResponse ,event , mainObject)
    {
        try
        {
            mainObject.m_Log.Write("nsCoxMailPOP.js - emailOnloadHandler - START");
            
            var httpChannel = event.QueryInterface(Components.interfaces.nsIHttpChannel);
            //check status should be 200.
            if (httpChannel.responseStatus != 200)
                throw new Error("error status " + httpChannel.responseStatus);

            var szContentType =  httpChannel.getResponseHeader("Content-Type");
            mainObject.m_Log.Write("nsCoxMailPOP.js - emailOnloadHandler - szContentType "+szContentType);
            if (szContentType.search(/text\/html/i)!=-1)
               throw new Error("Unexpected reply from CoxMail; " + szResponse);

            var szPOPResponse = "+OK " + (szResponse.length + 22) + "\r\n" + "X-WebMail: true\r\n" + szResponse + "\r\n.\r\n";
            mainObject.serverComms(szPOPResponse);

            mainObject.m_Log.Write("nsCoxMailPOP.js - emailOnloadHandler - END");
        }
        catch(err)
        {
            mainObject.m_Log.DebugDump("nsCoxMailPOP.js: emailOnloadHandler : Exception : "
                                              + err.name
                                              + ".\nError message: "
                                              + err.message);
            mainObject.serverComms("-ERR negative vibes (CoxMail)\r\n");
        }
    },



    //dele
    deleteMessage : function(msgIndex)
    {
        try
        {
            this.m_Log.Write("nsCoxMailPOP.js - deleteMessage - START");

            if ( this.m_bAuthorised == false ) return false;

            //get msg id
            this.m_szMsgID = this.m_aCoxMailPopMessages[msgIndex-1].szMessageID;
            this.m_Log.Write("nsCoxMailPOP.js - deleteMessage - msg id: " + this.m_szMsgID);
    
            this.m_HttpComms.addValuePair('msgIds', this.m_szMsgID);
            this.m_HttpComms.addValuePair('destfid', '');
            this.m_HttpComms.addValuePair('inbox_create_folder', '');
            this.m_HttpComms.addValuePair('newSubjectMsgId', '');
            this.m_HttpComms.addValuePair('newSubject', '');
            this.m_HttpComms.addValuePair('email', '');
    
            this.m_HttpComms.setURI(COX_MAIL_DELETE_URL);
            this.m_HttpComms.setRequestMethod("POST");

            var bResult = this.m_HttpComms.send(this.deleteMessageOnloadHandler, this);
            this.m_iStage = 0;
    
            this.m_Log.Write("nsCoxMailPOP.js - deleteMessage - END");
            return true;
        }
        catch(e)
        {
            this.m_Log.DebugDump("nsCoxMailPOP.js: deleteMessage : Exception : "
                                          + e.name +
                                          ".\nError message: "
                                          + e.message);
            return false;
        }
    },


    deleteMessageOnloadHandler : function (szResponse ,event , mainObject)
    {
        try
        {
            mainObject.m_Log.Write("nsCoxMailPOP.js - deleteMessageOnload - START");
    
            var httpChannel = event.QueryInterface(Components.interfaces.nsIHttpChannel);
            //check status should be 200.
            mainObject.m_Log.Write("nsCoxMailPOP.js - deleteMessageOnload :" + httpChannel.responseStatus);
            if (httpChannel.responseStatus != 200 )
                throw new Error("error status " + httpChannel.responseStatus);
    
            mainObject.serverComms("+OK its history\r\n");
            mainObject.m_Log.Write("nsCoxMailPOP.js - deleteMessageOnload - END");
        }
        catch(e)
        {
            mainObject.m_Log.DebugDump("nsCoxMailPOP.js: deleteMessageOnload : Exception : " + e.name
                                                  + ".\nError message: "
                                                  + e.message);
            mainObject.serverComms("-ERR negative vibes (CoxMail)\r\n");
        }
    },

    //cookies are deleted when the connection ends so i dont need to download pages
    logOut : function()
    {
        try
        {
            this.m_Log.Write("nsCoxMailPOP.js - logOUT - START");

            if ( this.m_bReUseSession)
            {
                this.m_Log.Write("nsCoxMailSMTP.js - logOUT - Saving session Data");
                this.m_ComponentManager.addElement(this.m_szUserName, "bSessionStored", true);
            }
            else
            {
                var oCookies = Components.classes["@mozilla.org/nsWebMailCookieManager2;1"]
                                         .getService(Components.interfaces.nsIWebMailCookieManager2);
                oCookies.removeCookie(this.m_szUserName);

                this.m_ComponentManager.deleteAllElements(this.m_szUserName);
            }

            this.m_bAuthorised = false;
            this.serverComms("+OK Your Out\r\n");

            this.m_Log.Write("nsCoxMailPOP.js - logOUT - END");
            return true;
        }
        catch(e)
        {
            this.m_Log.DebugDump("nsCoxMailPOP.js: logOUT : Exception : "
                                      + e.name
                                      + ".\nError message: "
                                      + e.message);
            return false;
        }
    },

    serverComms : function (szMsg)
    {
        try
        {
            this.m_Log.Write("nsCoxMailPOP.js - serverComms - START");
            this.m_Log.Write("nsCoxMailPOP.js - serverComms msg " + szMsg);
            var iCount = this.m_oResponseStream.write(szMsg,szMsg.length);
            this.m_Log.Write("nsCoxMailPOP.js - serverComms sent count: " + iCount +" msg length: " +szMsg.length);
            this.m_Log.Write("nsCoxMailPOP.js - serverComms - END");
        }
        catch(e)
        {
            this.m_Log.DebugDump("nsCoxMailPOP.js: serverComms : Exception : "
                                              + e.name
                                              + ".\nError message: "
                                              + e.message);
        }
    },


/******************************************************************************/
/***************** XPCOM  stuff ***********************************************/
/******************************************************************************/
    QueryInterface : function (iid)
    {
        if (!iid.equals(Components.interfaces.nsIPOPDomainHandler)
                                      && !iid.equals(Components.interfaces.nsISupports))
            throw Components.results.NS_ERROR_NO_INTERFACE;

        return this;
    }
}


/******************************************************************************/
/* FACTORY*/
var nsCoxMailFactory = new Object();

nsCoxMailFactory.createInstance = function (outer, iid)
{
    if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;

    if (!iid.equals(nsCoxMailPOPClassID) && !iid.equals(Components.interfaces.nsISupports))
        throw Components.results.NS_ERROR_INVALID_ARG;

    return new nsCoxMail();
}


/******************************************************************************/
/* MODULE */
var nsCoxMailModule = new Object();

nsCoxMailModule.registerSelf = function(compMgr, fileSpec, location, type)
{
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    compMgr.registerFactoryLocation(nsCoxMailPOPClassID,
                                    "CoxMailComponent",
                                    nsCoxMailPOPContactID,
                                    fileSpec,
                                    location,
                                    type);
}


nsCoxMailModule.unregisterSelf = function(aCompMgr, aFileSpec, aLocation)
{
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(nsCoxMailPOPClassID, aFileSpec);
}


nsCoxMailModule.getClassObject = function(compMgr, cid, iid)
{
    if (!cid.equals(nsCoxMailPOPClassID))
        throw Components.results.NS_ERROR_NO_INTERFACE;

    if (!iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    return nsCoxMailFactory;
}


nsCoxMailModule.canUnload = function(compMgr)
{
    return true;
}
/******************************************************************************/


function NSGetModule(compMgr, fileSpec)
{
    return nsCoxMailModule;
}
